/*
 *  INFO:
 *     tap.cpp (C) 2004  Dr. Yuri Klimets (www.jtag.tk, jtagtools.sf.net)  
 *     E-mail: klimets@jtag.tk
 *     Rev. 1.2 - 30.07.2002
 *  
 *  
 *  DESCRIPTION:
 * Contains implementation of TAP class methods.
 * For more detailed info - see "tap.h"
 *
 *
 *  DISCLAIMER:
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/  

#include "tap.h"

int DP=0x378;           // data port
int SP=0x379;           // status port
int CP=0x37a;           // control port

TAP::TAP(int port, int g_size, int mode=DEV_LP) {
 MODE=mode;
 status=0;
 DP=port;SP=port+1;CP=port+2;
 oldv=0;
 FD=0;
 g_pos=0;
 status=TAP_SUCCESS;
 switch (MODE) {

    case RAW_ACCESS : 
            if (ioperm(DP,3,1)<0) {status=TAP_DENIED; break;}
            SET_CTRL();
            break;

    case DEV_PORT :
            FD=open("/dev/port",O_RDWR);
            if (FD<0) {status=TAP_FAILED; break;}
            SET_CTRL();
            break;

    case DEV_LP : 
            FD=open("/dev/lp0",O_WRONLY);
            if (FD<0) {status=TAP_FAILED; break;}
            ioctl(FD,LPABORT,1);
            ioctl(FD,LPWAIT,0);
            SET_CTRL();
            break;

    default : 
            status=TAP_UNSUPP; break;

 }
 if (status==TAP_SUCCESS) {
    g_buf=NULL;
 g_buf=new BYTE[g_size+1];
    if (g_buf==NULL) status=TAP_MEMERR;
 }
}


TAP::~TAP() {
 switch (MODE) {
    case RAW_ACCESS  : break;
    case DEV_PORT    : close(FD);break;
    case DEV_LP      : close(FD);break;
    default          : break;
 }
 if (g_buf!=NULL) {delete[] g_buf; g_buf=NULL;}
}

void TAP::update() {g_buf[g_pos++]=oldv;}

void TAP::realize() {
 DWORD i=0;
 switch (MODE) {

   case RAW_ACCESS : 
            for (i=0;i<g_pos;i++) outb(g_buf[i],DP);
            g_pos=0;
            break;

   case DEV_PORT : 
            for (i=0;i<g_pos;i++) {
               lseek(FD,DP,SEEK_SET);
               write(FD,g_buf+i,1);
            }
            g_pos=0; 
            break;

   case DEV_LP : 
            write(FD,g_buf,g_pos);
            g_pos=0;
            break;
 }
}

void TAP::set_bit(int bit, int val) {
 BYTE mask=0x01;
 mask<<=bit;
 oldv=(BYTE)(val)?(oldv|mask):(oldv&(0xff^mask));
}

BYTE TAP::get_bit(int bit)
{
 unsigned int s;
 switch (MODE) {
   case RAW_ACCESS : s=inb(SP);break;
   case DEV_PORT   : lseek(FD,SP,SEEK_SET);read(FD,&s,1);break;
   case DEV_LP     : ioctl(FD,LPGETSTATUS,&s);         
                     if (s&LP_PSELECD) s=0xff;
                       else s=0;
                     break;
 }
 byte mask=0x01;
 mask<<=bit;
 return (BYTE)((s&mask)?1:0);
}

void TAP::TCK_CLOCK(int NUM=1) {
 if (NUM==1) {TCK(1);update();TCK(0);return;}
 for (int i=0;i<(NUM-1);i++) {
     TCK(1);update();TCK(0);update();
 }
 TCK(1);update();TCK(0);    
}

